import json
import os
import sys
import click


class Context(object):
    __CONFIG_FILE = '/etc/obsctl/obsauth.json'

    def __init__(self):
        if os.path.isfile(self.__CONFIG_FILE):
            self.get_configuration()
        else:
            self.prompt_for_configuration()

    def get_configuration(self):
        try:
            with open((self.__CONFIG_FILE), 'r') as obsauth_file:
                obsauth_filedata = obsauth_file.read()
                obsauth_data = json.loads(obsauth_filedata)
                obsauth_file.close()
                self.obs_server = obsauth_data.get('obs_server')
                self.obs_user = obsauth_data.get('obs_user')
                self.obs_pass = obsauth_data.get('obs_pass')
        except Exception as e:
            print('Error opening configuration file: {}'.format(e))

    def prompt_for_configuration(self):
        obs_server = click.prompt('OBS server',
                                  default='https://api.opensuse.org')
        obs_user = click.prompt('OBS login user', type=str)
        obs_pass = click.prompt('OBS login password', type=str,
                                hide_input=True)
        self.obs_server = obs_server
        self.obs_user = obs_user
        self.obs_pass = obs_pass

    def set_configuration(self, obs_server, obs_user, obs_pass):
        click.edit(filename=self.__CONFIG_FILE)
        # Now pull the proper configuration
        self.get_configuration()


pass_context = click.make_pass_decorator(Context, ensure=True)
cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                          'commands'))


class CLI(click.MultiCommand):

    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(cmd_folder):
            if filename.endswith('.py') and \
               filename.startswith('cmd_'):
                rv.append(filename[4:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            if sys.version_info[0] == 2:
                name = name.encode('ascii', 'replace')
            mod = __import__('obsctl.commands.cmd_' + name,
                             None, None, ['cli'])
        except ImportError:
            return
        return mod.cli


@click.command(cls=CLI)
@pass_context
def cli(ctx):
    """An OBS command line interface."""
